/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2;

import dk.sds.ecpr2.security.SoapHeadersEndpointInterceptor;
import dk.sds.ecpr2.util.CustomValidationInterceptor;
import dk.sds.ecpr2.util.TimeHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.ClassPathResource;
import org.springframework.ws.config.annotation.WsConfigurerAdapter;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.wsdl.WsdlDefinition;
import org.springframework.ws.wsdl.wsdl11.DefaultWsdl11Definition;
import org.springframework.xml.xsd.SimpleXsdSchema;
import org.springframework.xml.xsd.XsdSchema;

import javax.annotation.PostConstruct;
import java.util.Date;
import java.util.List;
import java.util.Properties;

@SpringBootApplication
@Log4j2
public class ECPR2Application extends WsConfigurerAdapter {

    @Autowired
    dk.sds.ecpr2.util.Properties properties;

    public static Date DEPLOY_TIME = TimeHelper.getCurrentTime();

    public static void main(String[] args) {
        SpringApplication.run(ECPR2Application.class, args);
    }

    @Bean
    public WsdlDefinition serviceDefinition_20171001(XsdSchema schema) {
        final DefaultWsdl11Definition bean = new DefaultWsdl11Definition();
        bean.setSchema(schema);
        bean.setPortTypeName("PersonalDataCardPort");
        final Properties soapActions = new Properties();
        soapActions.put("GetPersonalDataCard",      "http://sds.dk/ecpr2/2017/10/01/GetPersonalDataCard");
        soapActions.put("UpdateContactInformation", "http://sds.dk/ecpr2/2017/10/01/UpdateContactInformation");
        bean.setSoapActions(soapActions);
        return bean;
    }

    @Bean
    public XsdSchema schema() {
        return new SimpleXsdSchema(new ClassPathResource("schema/2017_10_01/ecpr2service.xsd"));
    }

    @Override
    public void addInterceptors(List<EndpointInterceptor> interceptors) {
        final CustomValidationInterceptor validatingInterceptor = new CustomValidationInterceptor(properties);
        validatingInterceptor.setValidateRequest(properties.schemavalidationValidateRequests);
        validatingInterceptor.setValidateResponse(properties.schemavalidationValidateResponse);
        validatingInterceptor.setXsdSchema(schema());
        interceptors.add(validatingInterceptor);
        interceptors.add(new SoapHeadersEndpointInterceptor());
        super.addInterceptors(interceptors);
    }

    @PostConstruct
    public void logSettings() {
        log.info("Info logging is enabled.");
        log.debug("Debug logging is enabled.");
        log.trace("Trace logging is enabled.");
    }

}
