/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.domain;

import lombok.Builder;
import lombok.Data;

/**
 * Data Access Object for ServiceActor table
 */

@Data
@Builder
public class ServiceActorVO {
    private Long serviceActorPID;
    private String personIdentifier;
    private String personIdentifierSource;
    private String firstName;
    private String lastName;
    private String authorisationNumber;
    private String educationCode;
    private String specialityCode;
    private String specialityCodeSource;
    private String organizationId;
    private String organizationIdSource;
    private String organizationName;

    public String getName() {
        return getFirstName() + " " + getLastName();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final ServiceActorVO that = (ServiceActorVO) o;

        if (personIdentifier != null ? !personIdentifier.equals(that.personIdentifier) : that.personIdentifier != null)
            return false;
        if (personIdentifierSource != null ? !personIdentifierSource.equals(that.personIdentifierSource) : that.personIdentifierSource != null)
            return false;
        if (firstName != null ? ! firstName.equals(that.firstName) : that.firstName != null) return false;
        if (lastName != null ? !lastName.equals(that.lastName) : that.lastName != null) return false;
        if (authorisationNumber != null ? !authorisationNumber.equals(that.authorisationNumber) : that.authorisationNumber != null)
            return false;
        if (educationCode != null ? !educationCode.equals(that.educationCode) : that.educationCode != null)
            return false;
        if (specialityCode != null ? !specialityCode.equals(that.specialityCode) : that.specialityCode != null)
            return false;
        if (organizationId != null ? !organizationId.equals(that.organizationId) : that.organizationId != null)
            return false;
        if (organizationIdSource != null ? !organizationIdSource.equals(that.organizationIdSource) : that.organizationIdSource != null)
            return false;
        if (organizationName != null ? !organizationName.equals(that.organizationName) : that.organizationName != null)
            return false;

        return specialityCodeSource != null ? specialityCodeSource.equals(that.specialityCodeSource) : that.specialityCodeSource == null;

    }

    @Override
    public int hashCode() {
        int result = personIdentifier != null ? personIdentifier.hashCode() : 0;
        result = 31 * result + (personIdentifierSource != null ? personIdentifierSource.hashCode() : 0);
        result = 31 * result + (firstName != null ? firstName.hashCode() : 0);
        result = 31 * result + (lastName != null ? lastName.hashCode() : 0);
        result = 31 * result + (authorisationNumber != null ? authorisationNumber.hashCode() : 0);
        result = 31 * result + (educationCode != null ? educationCode.hashCode() : 0);
        result = 31 * result + (specialityCode != null ? specialityCode.hashCode() : 0);
        result = 31 * result + (specialityCodeSource != null ? specialityCodeSource.hashCode() : 0);
        result = 31 * result + (organizationId != null ? organizationId.hashCode() : 0);
        result = 31 * result + (organizationIdSource != null ? organizationIdSource.hashCode() : 0);
        result = 31 * result + (organizationName != null ? organizationName.hashCode() : 0);
        return result;
    }
}
