/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.security;

import org.springframework.ws.WebServiceMessage;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.soap.SoapMessage;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Fetch SOAP header information and store in RequestContext
 */
public class SoapHeadersEndpointInterceptor implements EndpointInterceptor {
    private static final int MAX_IDLE = 50;
    private static final int INIT_IDLE_CAPACITY = 10;

    //private static ObjectPool<Marshaller> marshallerPool = new StackObjectPool<>(new MarshallerFactory(), MAX_IDLE, INIT_IDLE_CAPACITY);
    //private static ObjectPool<Unmarshaller> unmarshallerPool = new StackObjectPool<>(new UnmarshallerFactory(), MAX_IDLE, INIT_IDLE_CAPACITY);

    @Override
    public boolean handleRequest(MessageContext messageContext, Object o) throws Exception {
        RequestContext context = RequestContext.get();
        context.reset();

        // TODO: logic below will be replaced by WSProxy format
        /*WebServiceMessage webServiceMessageRequest = messageContext.getRequest();
        if(webServiceMessageRequest != null) {
            SoapMessage soapMessage = (SoapMessage) webServiceMessageRequest;
            SoapHeader soapHeader = soapMessage.getSoapHeader();
            if(soapHeader != null) {
                Iterator<SoapHeaderElement> wsseHeaderIterator = soapHeader.examineHeaderElements(new QName("http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd", "Security"));
                if (wsseHeaderIterator.hasNext()) {
                    SoapHeaderElement wsseHeaderElement = wsseHeaderIterator.next();
                    Security securityHeader = unmarshallerPool.borrowObject().unmarshal(wsseHeaderElement.getSource(), Security.class).getValue();
                    if (securityHeader != null) {
                        context.setUserInfo(new UserInfo(securityHeader));
                    }
                }

                Iterator<SoapHeaderElement> medcomHeaderIterator = soapHeader.examineHeaderElements(new QName("http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", "Header"));
                if (medcomHeaderIterator.hasNext()) {
                    SoapHeaderElement medcomHeaderElement = medcomHeaderIterator.next();
                    Header medcomHeader = unmarshallerPool.borrowObject().unmarshal(medcomHeaderElement.getSource(), Header.class).getValue();
                    if (medcomHeader != null && medcomHeader.getLinking() != null) {
                        context.setMessageId(medcomHeader.getLinking().getMessageID());
                        context.setFlowId(medcomHeader.getLinking().getFlowID());
                        context.setRequestType(RequestContext.RequestType.DGWS);
                    }
                }

                // TODO: Handle IDWS
            }
        }*/
        return true;
    }


    @Override
    public boolean handleResponse(MessageContext messageContext, Object o) throws Exception {
        if (!RequestContext.get().getRequestType().equals(RequestContext.RequestType.PLAIN)) {
            addOutgoingHeaders(messageContext, false);
        }
        return true;
    }

    @Override
    public boolean handleFault(MessageContext messageContext, Object o) throws Exception {
        if (!RequestContext.get().getRequestType().equals(RequestContext.RequestType.PLAIN)) {
            addOutgoingHeaders(messageContext, false);
        }
        return true;
    }

    @Override
    public void afterCompletion(MessageContext messageContext, Object o, Exception e) throws Exception {
    }

    private void addOutgoingHeaders(MessageContext messageContext, boolean fault) throws Exception {
        WebServiceMessage webServiceMessageResponse = messageContext.getResponse();
        SoapMessage soapMessage = (SoapMessage) webServiceMessageResponse;
        NodeList headerNodes = soapMessage.getDocument().getElementsByTagNameNS("http://schemas.xmlsoap.org/soap/envelope/", "Header");
        if (headerNodes != null && headerNodes.getLength() == 1) {
            Node headerNode = headerNodes.item(0);
            RequestContext context = RequestContext.get();

            switch (context.getRequestType()) {
                case DGWS:
                    // TODO: logic below will be replaced by WSProxy format
                    /*Security securityHeader = new Security();
                    Timestamp timestamp = new Timestamp();
                    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
                    cal.set(Calendar.MILLISECOND, 0);
                    timestamp.setCreated(cal);
                    securityHeader.setTimestamp(timestamp);
                    marshallerPool.borrowObject().marshal(securityHeader, headerNode);

                    Header medcomHeader = new Header();
                    medcomHeader.setFlowStatus(FlowStatusValues.FLOW_FINALIZED_SUCCESFULLY);
                    Linking linking = new Linking();
                    linking.setMessageID(UUID.randomUUID().toString());
                    linking.setInResponseToMessageID(context.getMessageId());
                    linking.setFlowID(context.getFlowId());
                    medcomHeader.setLinking(linking);
                    marshallerPool.borrowObject().marshal(medcomHeader, headerNode);*/

                    break;

                case IDWS:
                    // TODO Handle IDWS
                    break;
            }
        } else {
            throw new RuntimeException("Headerelement not found"); // TODO: Define new Exception?
        }
    }
}
