/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.service;

import dk.sds.ecpr2.dao.*;
import dk.sds.ecpr2.dao.exceptions.DAOException;
import dk.sds.ecpr2.domain.ContactInformationVO;
import dk.sds.ecpr2.domain.PersonalDataCardVO;
import dk.sds.ecpr2.domain.ServiceActorVO;
import dk.sds.ecpr2.util.FaultCode;
import dk.sds.ecpr2.util.FaultUtil;
import dk.sds.ecpr2.util.TimeHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
@Log4j2
public class ECPR2ServiceImpl implements ECPR2Service {

    @Autowired
    private ContactInformationDAO contactInformationDAO;

    private static final Date VALID_TO_DEFAULT = TimeHelper.createNew(8099, 0, 1);
    private static final int INITIAL_VERSION_DEFAULT = 1;

    @Override
    public PersonalDataCardVO getPersonalDataCard(String cprnr) {
        /*try {
            final PersonalDataCardVO pdc = personalDataCardDAO.getCurrentPersonalDataCard(cprnr);
            if (pdc != null) {

                final Date now = TimeHelper.getCurrentTime();

                final ContactInformationVO contactInformation = contactInformationDAO.getContactInformation(cprnr, now);
                if (contactInformation != null) {
                    pdc.setContactInformation(contactInformation);
                }
                pdc.setRelatives(relativeDAO.getRelatives(cprnr, now));
                pdc.setTempAddresses(tempAddressDAO.getTempAddresses(cprnr, now));
                pdc.setLanguages(languageDAO.getLanguages(cprnr, now));
            }
            return pdc;
        } catch (DAOException e) {
            FaultUtil.throwECPR2Fault(FaultCode.GET_PERSONAL_DATA_CARD_ERROR, e);
        }*/
        return null;
    }

    @Override
    public void updateContactInformation(ContactInformationVO contactInfo, String cpr, ServiceActorVO actor) {
        final Date nowTimestamp = TimeHelper.getCurrentTime();

        try {
            final ContactInformationVO currentVersionOfContactInfo = contactInformationDAO.getContactInformationWithoutDataCard(cpr, nowTimestamp);
            final long newCardId = createNewVersionOfPersonalDataCard(cpr, actor);

            int nextVersion = INITIAL_VERSION_DEFAULT;

            if (currentVersionOfContactInfo != null) {
                final int currentVersion = currentVersionOfContactInfo.getVersion();
                final long currentCardId = currentVersionOfContactInfo.getPersonalDataCardPID();
                nextVersion = currentVersion + 1;
                contactInformationDAO.updateValidToTimestamp(currentCardId, currentVersion, nowTimestamp);
            }

            contactInfo.setPersonalDataCardPID(newCardId);
            contactInfo.setVersion(nextVersion);
            contactInfo.setValidFrom(nowTimestamp);
            contactInfo.setValidTo(VALID_TO_DEFAULT);

            contactInformationDAO.createContactInformation(contactInfo);

        } catch (DAOException e) {
            FaultUtil.throwECPR2Fault(FaultCode.UPDATE_CONTACT_INFO_INTERNAL_SERVER_ERROR, e);
        }
    }

    /*private long createPersonalDataCardAndActor(PersonalDataCardVO dataCard) throws DAOException {

        Long serviceActorPID = serviceActorDAO.getServiceActorFromHashCode(dataCard.getServiceActor());
        if (serviceActorPID == null) {
            serviceActorPID = serviceActorDAO.createServiceActor(dataCard.getServiceActor());
        }
        dataCard.getServiceActor().setServiceActorPID(serviceActorPID);

        final int createdId = personalDataCardDAO.createPersonalDataCard(dataCard);

        return createdId;
    }*/

    private long createNewVersionOfPersonalDataCard(String cpr, ServiceActorVO actor) throws DAOException {
        /*final PersonalDataCardVO existingCard = personalDataCardDAO.getCurrentPersonalDataCard(cpr);
        final int existingCardVersion = existingCard == null ? 0 : existingCard.getVersion();
        final int nextCardVersion = existingCardVersion + 1;

        final PersonalDataCardVO newDataCard = PersonalDataCardVO.builder()
                .personIdentifier(cpr)
                .version(nextCardVersion)
                .versionDescription("Version " + nextCardVersion + ", updated by " + actor.getName())
                .personIdentifierSource(AssigningAuthorities.CPR)
                .build();
        newDataCard.setServiceActor(actor);
        newDataCard.setModified(TimeHelper.getCurrentTime());

        return createPersonalDataCardAndActor(newDataCard);*/
        return 0;
    }


}
