/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.util;

import lombok.extern.log4j.Log4j2;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.server.endpoint.interceptor.PayloadValidatingInterceptor;
import org.xml.sax.SAXParseException;

@Log4j2
public class CustomValidationInterceptor extends PayloadValidatingInterceptor {

    private Properties properties;

    public CustomValidationInterceptor(Properties properties) {
        this.properties = properties;
    }

    @Override
    protected boolean handleResponseValidationErrors(MessageContext messageContext, SAXParseException[] errors) {
        SAXParseException[] origErrors = errors;
        int errorLength = errors.length;

        for (int i = 0; i < errorLength; ++i) {
            SAXParseException error = origErrors[i];
            logger.error("XML validation error on response: " + error.getMessage());
        }

        if (properties.schemavalidationFailOnResponseError) {
            throw new RuntimeException("Error: ECPR2 generated a schema-invalid response");
        } else {
            return true;
        }
    }

}