/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.util;

import dk.sds.ecpr2._2017._10._01.DGWSFault;
import dk.sds.ecpr2.security.RequestContext;
import lombok.extern.log4j.Log4j2;

/**
 * Throw DGWS fault from service depending on call type DGWS/IDWS
 */
@Log4j2
public class FaultUtil {
    private static final String DGWS_ERROR = "DGWS error";
    private static final String IDWS_ERROR = "IDWS error";

    public static void throwECPR2Fault(FaultCode faultCode) {
        throw new ECPRException(faultCode);
    }

    public static void throwECPR2Fault(FaultCode faultCode, Throwable cause) {
        throw new ECPRException(faultCode, cause);
    }

    public static void throwECPR2Fault(FaultCode faultCode, String faultInfo, Throwable cause) {
        throw new ECPRException(faultCode, faultInfo, cause);
    }

    public static void throwECPR2Fault(FaultCode faultCode, String faultInfo) {
        throw new ECPRException(faultCode, faultInfo, null);
    }

    public static void throwFault(FaultCode faultCode) throws Exception {
        throwFault(faultCode, null, null);
    }


    public static void throwFault(FaultCode faultCode, Throwable t) throws Exception {
        throwFault(faultCode, null, t);
    }

    public static void throwFault(FaultCode faultCode, String faultInfo, Throwable t) throws Exception {
        switch (RequestContext.get().getRequestType()) {
            case DGWS:
                log.error("Throwing dgws fault: " + faultCode + (faultInfo != null ? ", " + faultInfo : ""), t);
                throw new DGWSFault(faultCode.toString(), DGWS_ERROR + (faultInfo != null ? ": " + faultInfo : ""), t);
            case IDWS:
                // TODO: Handle IDWS
                log.error("Throwing idws fault: " + faultCode + (faultInfo != null ? ", " + faultInfo : ""), t);
                throw new RuntimeException(faultCode.toString() + ", " + IDWS_ERROR + (faultInfo != null ? ": " + faultInfo : ""), t);
            default:
                log.error("Throwing fault: " + faultCode + (faultInfo != null ? ", " + faultInfo : ""), t);
                throw new RuntimeException(faultCode.toString() + ", " + (faultInfo != null ? faultInfo : ""), t);
        }
    }
}
