/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2.webservice;

import dk.sds.ecpr2._2017._10._01.*;
import dk.sds.ecpr2.service.ECPR2Service;
import dk.sds.ecpr2.util.ECPRException;
import dk.sds.ecpr2.util.FaultCode;
import dk.sds.ecpr2.util.FaultUtil;
import dk.sds.ecpr2.webservice.exceptions.ParseException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;


@Endpoint
@Log4j2
public class ECPR2WebServiceImpl implements dk.sds.ecpr2.webservice.ECPR2WebService {

    @Autowired
    private ECPR2Service businessService;

    @Override
    @ResponsePayload
    public GetPersonalDataCardResponse getPersonalDataCard(@RequestPayload GetPersonalDataCardRequest request) throws Exception {
        final GetPersonalDataCardResponse response = new GetPersonalDataCardResponse();

        /*try {
            if (request != null && request.getId() != null) {
                final String cprnr = request.getId().getExtension();
                if (cprnr != null && cprnr.length() == 10) {

                    final PersonalDataCardVO pdc = businessService.getPersonalDataCard(cprnr);

                    final POCDMT000040ClinicalDocument clinicalDocument = toHL7CDAMapper.toClinicalDocument(cprnr, pdc);
                    toHL7CDAMapper.enrichClinicalDocumentWithCPREResponse(cprnr, clinicalDocument);
                    response.setClinicalDocument(clinicalDocument);
                }
            }
        } catch (Exception e) {
            commonExceptionHandler(e, FaultCode.GET_PERSONAL_DATA_CARD_ERROR, FaultCode.GET_PERSONAL_DATA_CARD_ERROR);
        }*/

        return response;
    }

    @Override
    @ResponsePayload
    public UpdateContactInformationResponse updateContactInformation(@RequestPayload UpdateContactInformationRequest request) throws Exception {
        final UpdateContactInformationResponse response = new ObjectFactory().createUpdateContactInformationResponse();

        /*try {
            new HL7CDAValidator().validate(request);
            final String cpr = request.getId().getExtension();
            final ServiceActorVO actor = fromHL7CDAMapper.createServiceActorFromRequest(request.getDataEnterer());
            final ContactInformationVO contactInformation = fromHL7CDAMapper.createContactInformationFromRequest(request.getTelecoms());
            businessService.updateContactInformation(contactInformation, cpr, actor);

        } catch (Exception e) {
            commonExceptionHandler(e, FaultCode.UPDATE_CONTACT_INFO_VALIDATION_ERROR, FaultCode.UPDATE_CONTACT_INFO_INTERNAL_SERVER_ERROR);
        }*/
        return response;
    }

    private void commonExceptionHandler(Exception e, FaultCode validationErrorCode, FaultCode internalServerErrorCode) throws Exception {
        if (e instanceof IllegalArgumentException || e instanceof ParseException) {
            FaultUtil.throwFault(validationErrorCode, e.getMessage(), e);
        } else if (e instanceof ECPRException) {
            final ECPRException ecprEx = (ECPRException) e;
            FaultUtil.throwFault(ecprEx.getFaultCode(), ecprEx.getMessage(), ecprEx);
        }

        FaultUtil.throwFault(internalServerErrorCode, e);
    }

}
