-- -----------------------------------------------------
-- Table `ecpr2`.`ServiceActor`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`ServiceActors` (
  `ServiceActorPID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær nøgle for serviceactor.',
  `PersonIdentifier` VARCHAR(10) NULL COMMENT 'Cpr nummer på den bruger, der har foretaget ændringen, hvis det kendes',
  `PersonIdentifierSource` VARCHAR(30) NULL COMMENT 'Type af personidentifier. Denne er “CPR” (hvis PersonIdentifier er et CPR-nr. I fremtiden kan der evt. stå “eCPR” hvis PersonIdentifier er et erstatnings-CPR-nr.\n',
  `FirstName` VARCHAR(80) NULL COMMENT 'Fornavn på actor',
  `LastName` VARCHAR(80) NULL COMMENT 'Efternavn på actor',
  `AuthorisationNumber` VARCHAR(5) NULL COMMENT 'Autorisationsnummer (sundhedsprofessionel)',
  `EducationCode` VARCHAR(30) NULL COMMENT 'Kode for uddannelse (sundhedsprofessionel)',
  `SpecialityCode` VARCHAR(10) NULL COMMENT 'Specialekode (sundhedsprofessionel)',
  `SpecialityCodeSource` VARCHAR(30) NULL COMMENT 'Specialekode (sundhedsprofessionel)',
  `OrganizationId` VARCHAR(80) NULL COMMENT 'ID for den organisation som actor agerer på vegne af.',
  `OrganizationIdSource` VARCHAR(30) NULL COMMENT 'Kan være SOR, SKS eller andre ID-typer.',
  `OrganizationName` VARCHAR(120) NULL COMMENT 'Navn for den organisation som actor agerer på vegne af.',
  `HashValue` INT NULL COMMENT 'Hashcode for værdier i record',
  PRIMARY KEY (`ServiceActorPID`),
  INDEX `HashValue_idx` (`HashValue` ASC))
  ENGINE = InnoDB
  COMMENT = 'Indeholder data vedr. den bruger, der har foretaget opdateringer til stamkortdata.';

-- -----------------------------------------------------
-- Table `ecpr2`.`PersonalDataCards`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`PersonalDataCards` (
  `PersonalDataCardPID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær dum nøgle for stamkortet',
  `PersonIdentifier` VARCHAR(10) NOT NULL COMMENT 'Cprnummer',
  `PersonIdentifierSource` VARCHAR(30) NOT NULL COMMENT 'Type af personidentifier. Denne er “CPR” (hvis PersonIdentifier er et CPR-nr. I fremtiden kan der evt. stå “eCPR” hvis PersonIdentifier er et erstatnings-CPR-nr.',
  `Version` INT NOT NULL COMMENT 'Version af stamkortet. Starter med 1 og tælles op med én hver gang stamkortet ændres.',
  `ModifiedDateTime` DATETIME(3) NOT NULL COMMENT 'Tidspunkt for oprettelse af denne version af stamkortet',
  `VersionDescription` VARCHAR(80) NULL COMMENT 'Årsagen til denne version af stamkortet. Dette kan med fordel være navn på den operation af servicen, der kaldes, f.eks. \"UpdateTempAddress\"',
  `ServiceActorPID` INT NOT NULL COMMENT 'Nøgle på bruger, der har oprettet denne version af stamkortet.',
  PRIMARY KEY (`PersonalDataCardPID`),
  INDEX `PersonIdentifier_idx` (`PersonalDataCardPID` ASC),
  CONSTRAINT `Pdc_PersonIdVersion_Unique` UNIQUE (`PersonIdentifier`, `Version`),
  CONSTRAINT `PdcServiceActorPID`
  FOREIGN KEY (`ServiceActorPID`) REFERENCES `ecpr2`.`ServiceActors` (`ServiceActorPID`))
  ENGINE = InnoDB
  COMMENT = 'Stamdata for en person givet ved cpr-nr.\n';

-- -----------------------------------------------------
-- Table `ecpr2`.`ContactInformations`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`ContactInformations` (
  `ContactInformationPID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær dum nøgle for kontaktinformation.',
  `PersonalDataCardPID` INT NOT NULL COMMENT 'ID på stamkort, som denne kontaktinformation er tilknyttet',
  `ValidFrom` DATETIME(3) NOT NULL COMMENT 'Gyldig fra dato.',
  `ValidTo` DATETIME(3) NOT NULL COMMENT 'Gyldig til dato. Når nye kontaktinformationer lagres, sættes gyldig til på den eksisterende og gyldig fra på den nye til nu, den nyes gyldig til sættes til en dato i fremtiden, f.eks. 01-01-2500.',
  `Version` INT NOT NULL COMMENT 'Versionsnummer på kontaktinformation, startende med 1. Tælles op hver gang kontaktinformation opdateres.',
  `MobilePhone` VARCHAR(45) NULL COMMENT 'Mobil telefonnummer',
  `HomePhone` VARCHAR(45) NULL COMMENT 'Privat telefonnummer',
  `WorkPhone` VARCHAR(45) NULL COMMENT 'Arbejdstelefonnummer',
  PRIMARY KEY (`ContactInformationPID`),
  INDEX `PersonalDataCardPID_idx` (`PersonalDataCardPID` ASC),
  CONSTRAINT `ContactInformationPdcPID`
  FOREIGN KEY (`PersonalDataCardPID`) REFERENCES `ecpr2`.`PersonalDataCards` (`PersonalDataCardPID`))
  ENGINE = InnoDB
  COMMENT = 'Indeholder aktuelle telefonnumre på den person, som stamkortet vedrører. Tabellen rummer historik udtrykt ved validFrom/To datointerval.';

-- -----------------------------------------------------
-- Table `ecpr2`.`Relatives`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`Relatives` (
  `RelativePID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær dum nøgle for record.',
  `PersonalDataCardPID` INT NOT NULL COMMENT 'ID på stamkort, som denne pårørende er tilknyttet',
  `Uuid` VARCHAR(36) NOT NULL COMMENT 'UUID til anvendelse ved opdateringer',
  `Version` INT NOT NULL COMMENT 'Versionsnummer på en pårørende, startende med 1. Tælles op hver gang samme pårørende (samme uuid) opdateres.',
  `ValidFrom` DATETIME(3) NOT NULL COMMENT 'Gyldig fra dato.',
  `ValidTo` DATETIME(3) NOT NULL COMMENT 'Gyldig til dato. Når den pårørende opdateres sættes gyldig til på den eksisterende record og gyldig fra på den nye record til nu, og den nyes gyldig til sættes til en dato i fremtiden, f.eks. 01-01-2500',
  `RelationshipType` VARCHAR(20) NOT NULL COMMENT 'Relationstype, f.eks. uspec_paaroerende, barn, aegtefaelle, forældre, barnebarn, svigerbarn, ingen_relationer, nabo, samboende, registreret_partner, søskende, øvrig_familie (MedCom)',
  `FirstName` VARCHAR(80) NULL COMMENT 'Fornavn og mellemnavne på pårørende',
  `LastName` VARCHAR(80) NULL COMMENT 'Efternavn på pårørende',
  `StreetAddressLine1` VARCHAR(80) NULL COMMENT 'Første adresselinje for pårørendes adresse',
  `StreetAddressLine2` VARCHAR(80) NULL COMMENT 'Anden adresselinje for pårørendes adresse',
  `StreetAddressLine3` VARCHAR(80) NULL COMMENT 'Tredje adresselinje for pårørendes adresse',
  `StreetAddressLine4` VARCHAR(80) NULL COMMENT 'Fjerde adresselinje for pårørendes adresse',
  `PostalCode` VARCHAR(10) NULL COMMENT 'Postnummer',
  `City` VARCHAR(80) NULL COMMENT 'Bynavn',
  `Country` VARCHAR(80) NULL COMMENT 'Land',
  `MobilePhone` VARCHAR(45) NULL COMMENT 'Mobil telefonnummer',
  `HomePhone` VARCHAR(45) NULL COMMENT 'Privat telefonnummer',
  `WorkPhone` VARCHAR(45) NULL COMMENT 'Arbejdstelefonnummer',
  `Note` VARCHAR(1000) NULL COMMENT 'Notat vedr. pårørende',
  PRIMARY KEY (`RelativePID`),
  INDEX `PersonalDataCardPID_idx` (`PersonalDataCardPID` ASC),
  CONSTRAINT `RelativesPdcPID`
  FOREIGN KEY (`PersonalDataCardPID`) REFERENCES `ecpr2`.`PersonalDataCards` (`PersonalDataCardPID`))
  ENGINE = InnoDB
  COMMENT = 'Pårørende, f.eks. mor, far, nabo mv.';

-- -----------------------------------------------------
-- Table `ecpr2`.`TempAddresses`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`TempAddresses` (
  `TempAddressPID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær dum nøgle for record',
  `PersonalDataCardPID` INT NOT NULL COMMENT 'ID på stamkort, som denne temporære adresserecord er tilknyttet',
  `Uuid` VARCHAR(36) NOT NULL COMMENT 'Uuid til anvendelse ved opdateringer',
  `Version` INT NOT NULL COMMENT 'Version. Starter ved 1 og forøges med 1 ved hver opdatering af record.',
  `ValidFrom` DATETIME(3) NOT NULL COMMENT 'Gyldig fra dato',
  `ValidTo` DATETIME(3) NOT NULL COMMENT 'Gyldig til dato. Når den temporære adresse opdateres sættes gyldig til på den eksisterende record og gyldig fra på den nye record til nu, og den nyes gyldig til sættes til en dato i fremtiden, f.eks. 01-01-2500',
  `StartingDate` DATE NULL COMMENT 'Dato, hvor den temporære adresse er gyldig fra.',
  `EndingDate` DATE NULL COMMENT 'Dato, hvor den temporære adresse er gyldig til',
  `StreetAddressLine1` VARCHAR(80) NULL COMMENT 'Første linje i adresse',
  `StreetAddressLine2` VARCHAR(80) NULL COMMENT 'Anden linje i adresse',
  `StreetAddressLine3` VARCHAR(80) NULL COMMENT 'Tredje linje i adresse',
  `StreetAddressLine4` VARCHAR(80) NULL COMMENT 'Fjerde linje i adresse',
  `PostalCode` VARCHAR(10) NULL COMMENT 'Postnummer',
  `City` VARCHAR(80) NULL COMMENT 'Bynavn',
  `Country` VARCHAR(80) NULL COMMENT 'Navn på land',
  PRIMARY KEY (`TempAddressPID`),
  INDEX `PersonalDataCardPID_idx` (`PersonalDataCardPID` ASC),
  CONSTRAINT `TempAddressesPdcPID`
  FOREIGN KEY (`PersonalDataCardPID`) REFERENCES `ecpr2`.`PersonalDataCards` (`PersonalDataCardPID`))
  ENGINE = InnoDB
  COMMENT = 'Midlertidig adresse for person';

-- -----------------------------------------------------
-- Table `ecpr2`.`Languages`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`Languages` (
  `LanguagePID` INT NOT NULL AUTO_INCREMENT COMMENT 'Primær dum nøgle for kontaktinformation.',
  `PersonalDataCardPID` INT NOT NULL COMMENT 'ID på stamkort, som denne language-record er tilknyttet',
  `Uuid` VARCHAR(36) NOT NULL COMMENT 'UUID til anvendelse ved opdateringer',
  `Version` INT NOT NULL COMMENT 'Versionsnummer på et sprog, startende med 1. Tælles op hver gang samme sprog (samme uuid) opdateres.',
  `ValidFrom` DATETIME(3) NOT NULL COMMENT 'Gyldig fra dato.',
  `ValidTo` DATETIME(3) NOT NULL COMMENT 'Gyldig til dato. Når nye sproginformationer lagres, sættes gyldig til på den eksisterende og gyldig fra på den nye til nu, og den nyes gyldig til sættes til en dato i fremtiden, f.eks. 01-01-2500.\n\n',
  `LanguageCode` CHAR(2) NOT NULL COMMENT 'Sprogkode fra ISO-639',
  PRIMARY KEY (`LanguagePID`),
  INDEX `PersonalDataCardPID_idx` (`PersonalDataCardPID` ASC),
  CONSTRAINT `LanguagesPdcPID`
  FOREIGN KEY (`PersonalDataCardPID`)
  REFERENCES `ecpr2`.`PersonalDataCards` (`PersonalDataCardPID`))
  ENGINE = InnoDB
  COMMENT = 'Indeholder talte sprog på den person, som stamkortet vedrører. Tabellen rummer historik udtrykt ved validFrom/To datointerval.';

-- -----------------------------------------------------
-- Table `ecpr2`.`LanguageCodes`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `ecpr2`.`LanguageCodes` (
  `LanguageCodePID` INT NOT NULL AUTO_INCREMENT,
  `BibliographicCode` CHAR(3) NULL,
  `TerminologicCode` CHAR(3) NULL,
  `LanguageCode` CHAR(2) NULL,
  `EnglishLanguageName` VARCHAR(100) NULL,
  `FrenchLanguageName` VARCHAR(100) NULL,
  PRIMARY KEY (`LanguageCodePID`),
  INDEX `IX_LanguageCode` (`LanguageCode` ASC))
  ENGINE = InnoDB
  COMMENT = 'ISO-639-2 sprogkoder';
