/**
 * MIT License
 *
 * Copyright (c) 2017 Trifork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sds.ecpr2;

import dk.sds.ecpr2._2017._10._01.PersonalDataCardPortType;
import dk.sds.ecpr2._2017._10._01.PersonalDataCardService;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;

import javax.xml.ws.BindingProvider;
import java.io.*;
import java.net.InetAddress;
import java.net.Socket;

public abstract class AbstractITCase {
    protected final static String HOST = "localhost";
    protected final static int PORT = 8085;
    protected final static String CONTEXT_PATH = "ecpr2";
    protected final static String ENDPOINT_URL_PREFIX = "http://" + HOST + ":" + PORT + "/" + CONTEXT_PATH + "/";

    private final String ENCODING = "UTF-8";
    private final int BUFFER_SIZE = 10000;

    private PersonalDataCardService service = new PersonalDataCardService();

    protected PersonalDataCardPortType getWebservicePort() {
        PersonalDataCardPortType port = service.getPersonalDataCardPort();
        String endpointURL = ENDPOINT_URL_PREFIX + "services/ecpr2";
        BindingProvider bp = (BindingProvider) port;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointURL);

        return port;
    }

    protected HttpResponse httpGet(String urlMapping) throws IOException {
        DefaultHttpClient httpClient = new DefaultHttpClient();
        HttpResponse response = httpClient.execute(new HttpGet(ENDPOINT_URL_PREFIX + urlMapping + "/"), new BasicHttpContext());

        return response;
    }

    protected String httpWrite(String urlMapping, String xmldata) throws IOException {
        InetAddress addr = InetAddress.getByName(HOST);
        Socket socket = new Socket(addr, PORT);

        //Send header
        BufferedWriter wr = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(), ENCODING));

        int length = xmldata.getBytes(ENCODING).length;
        wr.write("POST " + ENDPOINT_URL_PREFIX + urlMapping + " HTTP/1.1\r\n");
        wr.write("Host: " + HOST + ":" + PORT + "\r\n");
        wr.write("Content-Length: " + length + "\r\n");
        wr.write("Content-Type: text/xml;charset=" + ENCODING + "\r\n");
        wr.write("\r\n");

        //Send data
        wr.write(xmldata);
        wr.flush();

        // wait a bit (reading of response is a bit unstable)
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // Response
        StringBuilder b = new StringBuilder();
        BufferedReader rd = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        char[] buf = new char[BUFFER_SIZE];
        int len = BUFFER_SIZE;
        while (len == BUFFER_SIZE) {
            len = rd.read(buf, 0, BUFFER_SIZE);
            if (len > 0) {
                b.append(new String(buf, 0, len));
            }
        }

        wr.close();
        return b.toString();
    }
}
